@@ -2,7 +2,7 @@
 
 from functools import partial
 
-from sqlalchemy import tuple_
+from sqlalchemy import tuple_, and_, or_
 
 from .columns import find_order_key, parse_ob_clause
 from .results import Page, Paging, unserialize_bookmark
@@ -10,6 +10,7 @@ from .serial import InvalidPage
 from .sqla import (
     core_coerce_row,
     core_result_type,
+    get_bind,
     group_by_clauses,
     orm_coerce_row,
     orm_query_keys,
@@ -19,6 +20,36 @@ from .sqla import (
 
 PER_PAGE_DEFAULT = 10
 
+# Dialects built-in to sqlalchemy that support native tuple comparison.
+# Other custom dialects may support this too, but we err on the side of
+# breaking less.
+SUPPORTS_NATIVE_TUPLE_COMPARISON = ("postgresql", "mysql", "sqlite")
+
+
+def compare_tuples(lesser, greater, dialect=None):
+    """Given two sequences of equal length (whose entries can be SQL clauses or
+    simple values), create an SQL clause defining the lexicographic tuple
+    comparison ``lesser < greater``.
+
+    If ``dialect`` is provided and is an sqlalchemy SQL dialect supporting
+    native tuple comparison, the SQL emitted is a native tuple comparison.
+    Otherwise it is built manually using OR and AND."""
+    if len(lesser) != len(greater):
+        raise ValueError("Tuples must have same length to be compared!")
+    if len(lesser) == 1:
+        return lesser[0] < greater[0]
+    if dialect is not None and dialect.name.lower() in SUPPORTS_NATIVE_TUPLE_COMPARISON:
+        return tuple_(*lesser) < tuple_(*greater)
+    return or_(
+        *[
+            and_(
+                *[lesser[index] == greater[index] for index in range(eq_depth)],
+                lesser[eq_depth] < greater[eq_depth]
+            )
+            for eq_depth in range(len(lesser))
+        ]
+    )
+
 
 def where_condition_for_page(ordering_columns, place, dialect):
     """Construct the SQL condition required to restrict a query to the desired
@@ -41,11 +72,7 @@ def where_condition_for_page(ordering_columns, place, dialect):
     swapped = [c.pair_for_comparison(value, dialect) for c, value in zipped]
     row, place_row = zip(*swapped)
 
-    if len(row) == 1:
-        condition = row[0] > place_row[0]
-    else:
-        condition = tuple_(*row) > tuple_(*place_row)
-    return condition
+    return compare_tuples(greater=row, lesser=place_row, dialect=dialect)
 
 
 def orm_page_from_rows(
@@ -81,12 +108,8 @@ def perform_paging(q, per_page, place, backwards, orm=True, s=None):
             raise ValueError("Cannot page core selectable without a session/connection")
         selectable = q
         column_descriptions = q._raw_columns
-    try:
-        # for sessions, dialect is available via the bind:
-        dialect = s.get_bind(clause=getattr(q, 'statement', q)).dialect
-    except Exception:
-        # connections have a direct .dialect
-        dialect = s.dialect
+
+    dialect = get_bind(q, s).dialect
 
     order_cols = parse_ob_clause(selectable)
     if backwards:
